﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Threading;
using UnityEngine;

namespace Event
{
    /// <summary>
    /// EventDispose类提供了一个用于管理事件监听器的集中式事件处理系统。
    /// </summary>
    public static class EventDispose
    {
        /// <summary>
        /// 存储不同事件类型和ID的事件处理程序的字典。
        /// </summary>
        private static readonly ConcurrentDictionary<EventType, Dictionary<long, Delegate>> EventDic = new();

        /// <summary>
        /// 用于监听器的唯一标识符。
        /// </summary>
        private static long _listenerId = 0;

        /// <summary>
        /// 为指定的事件类型添加新的监听器。监听器与唯一标识符关联。
        /// </summary>
        /// <typeparam name="T">事件负载的类型。</typeparam>
        /// <param name="eventType">要监听的事件类型。</param>
        /// <param name="action">事件触发时要调用的操作。</param>
        /// <returns>包含事件类型和监听器的唯一标识符的元组。</returns>
        public static (EventType, long) AddListener<T>(EventType eventType, Action<T> action)
        {
            long key = Interlocked.Increment(ref _listenerId);
            // 获取或创建事件类型对应的字典
            var value = EventDic.GetOrAdd(eventType, _ => new());

            // 添加到内部字典
            value[key] = action;

            return (eventType, key);
        }

        /// <summary>
        /// 为指定的事件类型添加新的监听器。监听器与唯一标识符关联。
        /// </summary>
        /// <param name="eventType">要监听的事件类型。</param>
        /// <param name="action">事件触发时要调用的操作。</param>
        /// <returns>包含事件类型和监听器的唯一标识符的元组。</returns>
        public static (EventType, long) AddListener(EventType eventType, Action action)
        {
            long key = Interlocked.Increment(ref _listenerId);
            // 获取或创建事件类型对应的字典
            var value = EventDic.GetOrAdd(eventType, _ => new());

            // 添加到内部字典
            value[key] = action;

            return (eventType, key);
        }

        /// <summary>
        /// 将指定类型的事件广播给所有已注册的监听器。
        /// </summary>
        /// <param name="eventType">要广播的事件类型。</param>
        /// <param name="arg">要传递给事件监听器的参数。</param>
        /// <typeparam name="T">参数的类型。</typeparam>
        public static void Broadcast<T>(EventType eventType, T arg)
        {
            if (!EventDic.TryGetValue(eventType, out var value))
            {
                Debug.LogWarning($"不存在事件类型{eventType}的监听，无法广播事件");
                return;
            }

            foreach (var item in value)
            {
                if (item.Value is Action<T> action)
                {
                    action.Invoke(arg);
                }
                else
                {
                    Debug.LogError($"事件类型不匹配,请检查事件类型{eventType}的监听的参数类型是否和广播的参数类型一致");
                }
            }
        }

        /// <summary>
        /// 将指定类型的事件广播给所有已注册的监听器。
        /// </summary>
        /// <param name="eventType">要广播的事件类型。</param>
        public static void Broadcast(EventType eventType)
        {
            if (!EventDic.TryGetValue(eventType, out var value))
            {
                Debug.LogWarning($"不存在事件类型{eventType}的监听，无法广播事件");
                return;
            }

            foreach (var item in value)
            {
                if (item.Value is Action action)
                {
                    action.Invoke();
                }
                else
                {
                    Debug.LogError($"事件类型不匹配,请检查事件类型{eventType}的监听的参数类型是否和广播的参数类型一致");
                }
            }
        }

        /// <summary>
        /// 从指定的事件类型和键中删除监听器。
        /// </summary>
        /// <param name="eventType">要从中删除监听器的事件类型。</param>
        /// <param name="key">与监听器关联的唯一键。</param>
        public static void RemoveListener(EventType eventType, long key)
        {
            if (EventDic.TryGetValue(eventType, out var value) && value.ContainsKey(key))
            {
                value.Remove(key);
            }
        }

        /// <summary>
        /// 从指定的事件类型中删除监听器。
        /// </summary>
        /// <typeparam name="T">事件的类型。</typeparam>
        /// <param name="eventType">事件类型。</param>
        /// <param name="action">监听器操作。</param>
        public static void RemoveListener<T>(EventType eventType, Action<T> action)
        {
            if (EventDic.TryGetValue(eventType, out var value))
            {
                foreach (var item in value)
                {
                    if (item.Value.Equals(action))
                    {
                        value.Remove(item.Key);
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// 从指定的事件类型中删除监听器。
        /// </summary>
        /// <param name="eventType">事件类型。</param>
        /// <param name="action">监听器操作。</param>
        public static void RemoveListener(EventType eventType, Action action)
        {
            if (EventDic.TryGetValue(eventType, out var value))
            {
                foreach (var item in value)
                {
                    if (item.Value.Equals(action))
                    {
                        value.Remove(item.Key);
                        break;
                    }
                }
            }
        }
    }
}